import numpy as np
import matplotlib.pyplot as plt
from datetime import datetime as dt
import sys
from KEMP import Basic_FDTD, Dielectric, to_epr, to_SI, to_NU, Gold, Silver, PA
from KEMP import structures as stc


#----------------------------------general setting-------------------------------------------------------------------

nm = 1e-9    
um = 1e-6
dx, dy, dz = [10.*nm, 5*nm, 5*nm]
nx, ny, nz = [700, 200, 200]
x = np.ones(nx, dtype=np.float64)*dx 
y = np.ones(ny, dtype=np.float64)*dy 
z = np.ones(nz, dtype=np.float64)*dz 
lx, ly, lz = dx*nx, dy*ny, dz*nz
space_grid = (x, y ,z)
fdtd = Basic_FDTD('3D', space_grid, dtype=np.complex64, engine='nvidia_cuda')

pml_apply = {'x':'+-' , 'y':'+-'  , 'z':'+-'  }
pbc_apply = {'x':False, 'y':False, 'z':False}

save_name = 'examplehole'

#----------------------------------geometry configure-------------------------------------------------------------------

geometry_on = 1     #put in geometry if 1

geometry_x_thickness = 5000.*nm
geometry_y_thickness = 500.*nm
geometry_z_thickness = 500.*nm

diel1 = Dielectric(to_epr(fdtd, n=1.))
diel2 = Dielectric(to_epr(fdtd, n=10**20))


#-------------------------------others setting-------------------------------------------------

tmax = 30000

source_polarized = 'ey'

configure_wavelength =np.array([5, 15, 0.1])*um       # [first wavelength, final wavelength, gap of wavelengths]*(unit of wavelength)

#---------------------------------------------------------------------------------------------------


fdtd.apply_PML(pml_apply)
fdtd.apply_PBC(pbc_apply)

geometry_x_start = (lx-geometry_x_thickness)/2
geometry_x_end   = geometry_x_start+geometry_x_thickness
geometry_y_start = (ly-geometry_y_thickness)/2
geometry_y_end   = geometry_y_start+geometry_y_thickness
geometry_z_start = (lz-geometry_z_thickness)/2
geometry_z_end   = geometry_z_start+geometry_z_thickness

src_pos=np.int64((geometry_z_start - dz)/dz)
trs_pos=np.int64((geometry_z_start + geometry_z_end)/(2*dz))

pec = stc.Box(diel2, ((0.*um,0.*um,geometry_z_start),(lx+dx,ly+dy,geometry_z_end-dz/2)))
hole = stc.Box(diel1, ((geometry_x_start,geometry_y_start,geometry_z_start),(geometry_x_end-dx/2,geometry_y_end-dy/2,geometry_z_end-dz/2)))

if geometry_on == 1:
    structures = [pec,hole]
else :
    structures = []

inc    = fdtd.apply_direct_source(source_polarized, ((1,1,src_pos),(-2,-2,src_pos)))

from scipy.constants import c
wavelength = np.arange(configure_wavelength[0], configure_wavelength[1], configure_wavelength[2])
freqs = c/wavelength
freq0 = c/((configure_wavelength[1]+configure_wavelength[0])/2)
wfreq0_NU = to_NU(fdtd, 'angular frequency', 2.*np.pi*freq0)


rft_ex = fdtd.apply_RFT('ex', ((np.int64(geometry_x_start/dx),np.int64(geometry_y_start/dy),trs_pos), (np.int64(geometry_x_end/dx-dx/2),np.int64(geometry_y_end/dy-dy/2),trs_pos)), freqs)
rft_ey = fdtd.apply_RFT('ey', ((np.int64(geometry_x_start/dx),np.int64(geometry_y_start/dy),trs_pos), (np.int64(geometry_x_end/dx-dx/2),np.int64(geometry_y_end/dy-dy/2),trs_pos)), freqs)
rft_hx = fdtd.apply_RFT('hx', ((np.int64(geometry_x_start/dx),np.int64(geometry_y_start/dy),trs_pos), (np.int64(geometry_x_end/dx-dx/2),np.int64(geometry_y_end/dy-dy/2),trs_pos)), freqs)
rft_hy = fdtd.apply_RFT('hy', ((np.int64(geometry_x_start/dx),np.int64(geometry_y_start/dy),trs_pos), (np.int64(geometry_x_end/dx-dx/2),np.int64(geometry_y_end/dy-dy/2),trs_pos)), freqs)


fdtd.set_structures(structures)


print 'Setting Complete and READY to RUN'


plt.ion()


tc = 500
ts = 50.


t0 = dt.now()
for tstep in xrange(tmax):
    src = np.exp(-(tstep-tc)**2/(ts**2))*np.exp(1.j*wfreq0_NU*(tstep-tc)*fdtd.dt)
    #src = np.sin(wfreq0_NU*tstep*fdtd.dt)
    


    inc.set_source(src)
    fdtd.updateE()
    fdtd.updateH()

    if tstep % 1000 == 0:
        t1 = dt.now()
        print('%s %05d time step\r' % (t1-t0, tstep)),
        sys.stdout.flush()
        '''
        plt.clf()
        plt.plot(abs(fdtd.ex[nx/2,ny/2,:]*fdtd.hy[nx/2,ny/2,:]-fdtd.ey[nx/2,ny/2,:]*fdtd.hx[nx/2,ny/2,:]), 'r')
        plt.plot(abs(fdtd.ey[nx/2,ny/2,:]), 'b')
        plt.plot(abs(fdtd.hx[nx/2,ny/2,:]), 'k')
#        plt.plot(fdtd.cpwd.dr_fy_r[0][:,ny/2,nz/2], 'g')
#        plt.plot(fdtd.cpwd.cp_fy_r[0][:,ny/2,nz/2], 'b')
#        plt.plot(fdtd.cpwd.cp_fy_r[1][:,ny/2,nz/2], 'k')
        plt.ylim(-2., 2.)
        plt.draw()
#        plt.show()
        '''





t1 = dt.now()
print t1-t0, 'Simulation Complete'

dt_SI = to_SI(fdtd, 'time', fdtd.dt)
tsteps = np.arange(tmax, dtype=fdtd.dtype)
t = tsteps*dt_SI

ex_trs_ft = rft_ex.export()
ey_trs_ft = rft_ey.export()
hx_trs_ft = rft_hx.export()
hy_trs_ft = rft_hy.export()



sz = np.zeros(len(wavelength), dtype=np.complex64)

for i in range(ex_trs_ft.shape[0]):
    for j in range(ex_trs_ft.shape[1]):

        sz += ex_trs_ft[i,j]*np.conjugate(hy_trs_ft[i,j])-ey_trs_ft[i,j]*np.conjugate(hx_trs_ft[i,j])


import h5py as h5
f = h5.File('./save/%s.h5' %save_name,'w')

f.create_dataset('Sz', data=sz)

f.close()
